@@ -17,9 +17,7 @@ use Term::Form           qw();
 use Term::Form::ReadLine qw();
 
 use App::DBBrowser::Auxil;
-#use App::DBBrowser::CreateDropAttach::DropTable;   # required
 #use App::DBBrowser::Table::CommitWriteSQL          # required
-use App::DBBrowser::DB;
 use App::DBBrowser::GetContent;
 #use App::DBBrowser::Subqueries;                    # required
 
@@ -61,7 +59,7 @@ sub create_view {
         }
         $sql->{view_select_stmt} = $select_statment;
 
-        VIEW_NAME: while ( 1 ) {
+        VIEW_NAME: while ( 1 ) { ##
             $sql->{table} = '?';
             my $info = $ax->get_sql_info( $sql );
             # Readline
@@ -126,7 +124,7 @@ sub create_table {
         if ( exists $sf->{i}{ss}{$file_fs}{sheet_count} ) {
             $sheet_count = $sf->{i}{ss}{$file_fs}{sheet_count};
         }
-        my $table_name = ''; # table_name memory
+        my $table_name = ''; # table_name memory # n
 
         GET_TABLE_NAME: while ( 1 ) {
             $table_name = $sf->__get_table_name( $sql, $table_name, $count_table_name_loop ); # first time print_sql
@@ -222,14 +220,8 @@ sub create_table {
                                 # INSERT_DATA
                                 my $ok_insert = $sf->__insert_data( $sql );
                                 if ( ! $ok_insert ) {
-                                    require App::DBBrowser::CreateDropAttach::DropTable;
-                                    my $dt = App::DBBrowser::CreateDropAttach::DropTable->new( $sf->{i}, $sf->{o}, $sf->{d} );
-                                    my $drop_ok = $dt->__drop( $sql, 'table' );
-                                    if ( ! $drop_ok ) {
-                                        return;
-                                    }
-                                    $sf->{i}{stmt_types} = [ 'Create_table', 'Insert' ];
-                                    next GET_COLUMN_NAMES;
+                                    ##
+                                    return;
                                 }
                             }
                             if ( $sheet_count > 1 ) {
@@ -349,11 +341,11 @@ sub __get_column_names {
 
 sub __autoincrement_column {
     my ( $sf, $sql ) = @_;
-    my $plui = App::DBBrowser::DB->new( $sf->{i}, $sf->{o} );
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     $sf->{col_auto} = '';
-    if ( $sf->{constraint_auto} = $plui->primary_key_autoincrement_constraint( $sf->{d}{dbh} ) ) {
+    $sf->{constraint_auto} = $sf->__primary_key_autoincrement_constraint();
+    if ( $sf->{constraint_auto} ) {
         $sf->{col_auto} = $sf->{o}{create}{autoincrement_col_name};
     }
     if ( $sf->{col_auto} ) {
@@ -382,6 +374,43 @@ sub __autoincrement_column {
 }
 
 
+sub __primary_key_autoincrement_constraint {
+    # provide "primary_key_autoincrement_constraint" only if also "first_col_is_autoincrement" is available
+    my ( $sf ) = @_;
+    if ( $sf->{i}{driver} eq 'SQLite' ) {
+        return "INTEGER PRIMARY KEY";
+    }
+    if ( $sf->{i}{driver} =~ /^(?:mysql|MariaDB)\z/ ) {
+        return "INT NOT NULL AUTO_INCREMENT PRIMARY KEY";
+        # mysql: NOT NULL added automatically with AUTO_INCREMENT
+    }
+    if ( $sf->{i}{driver} eq 'Pg' ) {
+        my ( $pg_version ) = $sf->{d}{dbh}->selectrow_array( "SELECT version()" );
+        if ( $pg_version =~ /^\S+\s+([0-9]+)(?:\.[0-9]+)*\s/ && $1 >= 10 ) {
+            # since PostgreSQL version 10
+            return "INT GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY";
+        }
+        else {
+            return "SERIAL PRIMARY KEY";
+        }
+    }
+    if ( $sf->{i}{driver} eq 'Firebird' ) {
+        my ( $firebird_version ) = $sf->{d}{dbh}->selectrow_array( "SELECT RDB\$GET_CONTEXT('SYSTEM', 'ENGINE_VERSION') FROM RDB\$DATABASE" );
+        my $firebird_major_version = $firebird_version =~ s/^(\d+).+\z/$1/r;
+        if ( $firebird_major_version >= 4 ) {
+            return "INT GENERATED ALWAYS AS IDENTITY PRIMARY KEY";
+        }
+        elsif ( $firebird_major_version >= 3 ) {
+            return "INT GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY";
+        }
+    }
+    #if ( $sf->{i}{driver} eq 'DB2' ) {
+    #    #return "INT NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 1 INCREMENT BY 1) PRIMARY KEY";
+    #    return "INT NOT NULL GENERATED ALWAYS AS IDENTITY PRIMARY KEY";
+    #}
+}
+
+
 sub __edit_column_names {
     my ( $sf, $sql, $column_names ) = @_;
     my $tf = Term::Form->new( $sf->{i}{tf_default} );
