@@ -45,7 +45,8 @@ sub database_setting {
                 # Choose
                 my $idx_sec = $tc->choose(
                     $menu,
-                    { %{$sf->{i}{lyt_v}}, index => 1, default => $old_idx_sec, undef => '  <=' }
+                    { %{$sf->{i}{lyt_v}}, prompt => 'DB Settings for:',
+                      index => 1, default => $old_idx_sec, undef => '  <=' }
                 );
                 if ( ! defined $idx_sec || ! defined $menu->[$idx_sec] ) {
                     return;
