@@ -60,7 +60,7 @@ sub browse_the_table {
         my $tp = Term::TablePrint->new( $sf->{o}{table} );
         $tp->print_table(
             $all_arrayref,
-            { footer => "     '" . $sf->{d}{table} . "'     " }
+            { footer => $sf->{d}{table_footer} }
         );
 
         delete $sf->{o}{table}{max_rows}   if exists $sf->{o}{table}{max_rows};
